searchData={"items":[{"type":"module","title":"ExJSONPointer","doc":"An Elixir implementation of [RFC 6901](https://www.rfc-editor.org/rfc/rfc6901.html) JSON Pointer for locating specific values within JSON documents, and also supports [Relative JSON Pointer](https://datatracker.ietf.org/doc/html/draft-bhutton-relative-json-pointer-00) of the JSON Schema Specification draft-2020-12.","ref":"ExJSONPointer.html"},{"type":"module","title":"Usage - ExJSONPointer","doc":"The JSON pointer string syntax can be represented as a JSON string:\n\n```elixir\niex> ExJSONPointer.resolve(%{\"a\" => %{\"b\" => %{\"c\" => \"hello\"}}}, \"/a/b/c\")\n{:ok, \"hello\"}\n\niex> ExJSONPointer.resolve(%{\"a\" => %{\"b\" => %{\"c\" => \"hello\"}}}, \"/a/b\")\n{:ok, %{\"c\" => \"hello\"}}\n\niex> ExJSONPointer.resolve(%{\"a\" => %{\"b\" => %{\"c\" => [1, 2, 3]}}}, \"/a/b/c\")\n{:ok, [1, 2, 3]}\n\niex> ExJSONPointer.resolve(%{\"a\" => %{\"b\" => %{\"c\" => [1, 2, 3]}}}, \"/a/b/c/2\")\n{:ok, 3}\n\niex> ExJSONPointer.resolve(%{\"a\" => [%{\"b\" => %{\"c\" => [1, 2]}}, 2, 3]}, \"/a/2\")\n{:ok, 3}\n\niex> ExJSONPointer.resolve(%{\"a\" => [%{\"b\" => %{\"c\" => [1, 2]}}, 2, 3]}, \"/a/0/b/c/1\")\n{:ok, 2}\n```\n\nor a URI fragment identifier:\n\n```elixir\niex> ExJSONPointer.resolve(%{\"a\" => %{\"b\" => %{\"c\" => \"hello\"}}}, \"#/a/b/c\")\n{:ok, \"hello\"}\n\niex> ExJSONPointer.resolve(%{\"a\" => %{\"b\" => %{\"c\" => \"hello\"}}}, \"#/a/b\")\n{:ok, %{\"c\" => \"hello\"}}\n\niex> ExJSONPointer.resolve(%{\"a\" => %{\"b\" => %{\"c\" => [1, 2, 3]}}}, \"#/a/b/c\")\n{:ok, [1, 2, 3]}\n\niex> ExJSONPointer.resolve(%{\"a\" => %{\"b\" => %{\"c\" => [1, 2, 3]}}}, \"#/a/b/c/2\")\n{:ok, 3}\n\niex> ExJSONPointer.resolve(%{\"a\" => [%{\"b\" => %{\"c\" => [1, 2]}}, 2, 3]}, \"#/a/2\")\n{:ok, 3}\n\niex> ExJSONPointer.resolve(%{\"a\" => [%{\"b\" => %{\"c\" => [1, 2]}}, 2, 3]}, \"#/a/0/b/c/1\")\n{:ok, 2}\n```\n\nSome cases that a JSON pointer that references a nonexistent value:\n\n```elixir\niex> ExJSONPointer.resolve(%{\"a\" => %{\"b\" => %{\"c\" => \"hello\"}}}, \"/a/b/d\")\n{:error, \"not found\"}\n\niex> ExJSONPointer.resolve(%{\"a\" => %{\"b\" => %{\"c\" => [1, 2, 3]}}}, \"/a/b/c/4\")\n{:error, \"not found\"}\n\niex> ExJSONPointer.resolve(%{\"a\" => %{\"b\" => %{\"c\" => \"hello\"}}}, \"#/a/b/d\")\n{:error, \"not found\"}\n\niex> ExJSONPointer.resolve(%{\"a\" => %{\"b\" => %{\"c\" => [1, 2, 3]}}}, \"#/a/b/c/4\")\n{:error, \"not found\"}\n```\n\nSome cases that a JSON pointer has some empty reference tokens, and link a `$ref` [test case](https://github.com/json-schema-org/JSON-Schema-Test-Suite/blob/main/tests/draft2020-12/ref.json#L1023) from JSON Schema Test Suite(draft 2020-12) for reference.\n\n```elixir\niex> ExJSONPointer.resolve(%{\"\" => %{\"\" => 1}}, \"/\")\n{:ok, %{\"\" => 1}}\n\niex> ExJSONPointer.resolve(%{\"\" => %{\"\" => 1}}, \"//\")\n{:ok, 1}\n\niex> ExJSONPointer.resolve(%{\"\" => %{\"\" => 1, \"b\" => %{\"\" => 2}}}, \"//b\")\n{:ok, %{\"\" => 2}}\n\niex> ExJSONPointer.resolve(%{\"\" => %{\"\" => 1, \"b\" => %{\"\" => 2}}}, \"//b/\")\n{:ok, 2}\n\niex> ExJSONPointer.resolve(%{\"\" => %{\"\" => 1, \"b\" => %{\"\" => 2}}}, \"//b///\")\n{:error, \"not found\"}\n```\n\nInvalid JSON pointer syntax:\n\n```elixir\niex> ExJSONPointer.resolve(%{\"a\" =>%{\"b\" => %{\"c\" => [1, 2, 3]}}}, \"a/b\")\n{:error, \"invalid JSON pointer syntax\"}\n\niex> ExJSONPointer.resolve(%{\"a\" =>%{\"b\" => %{\"c\" => [1, 2, 3]}}}, \"##/a\")\n{:error, \"invalid JSON pointer syntax\"}\n\n```","ref":"ExJSONPointer.html#module-usage"},{"type":"module","title":"Relative JSON Pointer - ExJSONPointer","doc":"This library also supports [Relative JSON Pointer](https://datatracker.ietf.org/doc/html/draft-bhutton-relative-json-pointer-00) of the JSON Schema Specification draft-2020-12 which allows you to reference values relative to a specific location within a JSON document.\n\nA relative JSON pointer consists of:\n- A non-negative integer (prefix) that indicates how many levels up to traverse\n- An optional index manipulation (+N or -N) for array elements\n- An optional JSON pointer to navigate from the referenced location\n\n```elixir\n# Sample data\niex> data = %{\"foo\" => [\"bar\", \"baz\"], \"highly\" => %{\"nested\" => %{\"objects\" => true}}}\niex> ExJSONPointer.resolve(data, \"/foo/1\", \"0\") # Get the current value (0 levels up)\n{:ok, \"baz\"}\n# Get the parent array and access its first element (1 level up, then to index 0)\niex> ExJSONPointer.resolve(data, \"/foo/1\", \"1/0\")\n{:ok, \"bar\"}\n# Get the previous element in the array (current level, index - 1)\niex> ExJSONPointer.resolve(data, \"/foo/1\", \"0-1\")\n{:ok, \"bar\"}\n# Go up to the root and access a nested property\niex> ExJSONPointer.resolve(data, \"/foo/1\", \"2/highly/nested/objects\")\n{:ok, true}\n# Get the index of the current element in its array\niex> ExJSONPointer.resolve(data, \"/foo/1\", \"0#\")\n{:ok, 1}\n\n# Get the key name of a property in an object\niex> data2 = %{\"features\" => [%{\"name\" => \"environment friendly\", \"url\" => \"http://example.com\"}]}\niex> ExJSONPointer.resolve(data2, \"/features/0/url\", \"1/name\")\n{:ok, \"environment friendly\"}\niex> ExJSONPointer.resolve(data2, \"/features/0/url\", \"2#\")\n{:ok, \"features\"}\n```\n\nPlease see the test cases for more examples.","ref":"ExJSONPointer.html#module-relative-json-pointer"},{"type":"type","title":"ExJSONPointer.document/0","doc":"The JSON document to be processed, must be a map.","ref":"ExJSONPointer.html#t:document/0"},{"type":"type","title":"ExJSONPointer.pointer/0","doc":"The JSON Pointer string that follows RFC 6901 specification.\nCan be either a JSON String Representation (starting with '/') or\na URI Fragment Identifier Representation (starting with '#').","ref":"ExJSONPointer.html#t:pointer/0"},{"type":"function","title":"ExJSONPointer.resolve/2","doc":"Resolve the JSON document with the given JSON Pointer to find the accompanying value.\n\nThe pointer can be either:\n- An empty string (\"\") or \"#\" to reference the whole document\n- A JSON String Representation starting with \"/\"\n- A URI Fragment Identifier Representation starting with \"#\"","ref":"ExJSONPointer.html#resolve/2"},{"type":"function","title":"Examples - ExJSONPointer.resolve/2","doc":"iex> doc = %{\"foo\" => %{\"bar\" => \"baz\"}}\n    iex> ExJSONPointer.resolve(doc, \"/foo/bar\")\n    {:ok, \"baz\"}\n    iex> ExJSONPointer.resolve(doc, \"/foo/baz\")\n    {:error, \"not found\"}\n    iex> ExJSONPointer.resolve(doc, \"##foo\")\n    {:error, \"invalid JSON pointer syntax\"}","ref":"ExJSONPointer.html#resolve/2-examples"},{"type":"function","title":"ExJSONPointer.resolve/3","doc":"Resolve a relative JSON pointer from a starting position within a JSON document.\n\nThis function implements the Relative JSON Pointer specification as described in\n[draft-bhutton-relative-json-pointer-00](https://datatracker.ietf.org/doc/html/draft-bhutton-relative-json-pointer-00).\n\nA relative JSON pointer consists of:\n- A non-negative integer (prefix) that indicates how many levels up to traverse\n- An optional index manipulation (+N or -N) for array elements\n- An optional JSON pointer to navigate from the referenced location","ref":"ExJSONPointer.html#resolve/3"},{"type":"function","title":"Parameters - ExJSONPointer.resolve/3","doc":"- `document`: The JSON document to be processed\n- `start_json_pointer`: A JSON pointer that identifies the starting location within the document\n- `relative`: The relative JSON pointer to evaluate from the starting location","ref":"ExJSONPointer.html#resolve/3-parameters"},{"type":"function","title":"Examples - ExJSONPointer.resolve/3","doc":"iex> data = %{\"foo\" => [\"bar\", \"baz\"], \"highly\" => %{\"nested\" => %{\"objects\" => true}}}\n    iex> ExJSONPointer.resolve(data, \"/foo/1\", \"0\")\n    {:ok, \"baz\"}\n    iex> ExJSONPointer.resolve(data, \"/foo/1\", \"1/0\")\n    {:ok, \"bar\"}\n    iex> ExJSONPointer.resolve(data, \"/foo/1\", \"0-1\")\n    {:ok, \"bar\"}\n    iex> ExJSONPointer.resolve(data, \"/foo/1\", \"2/highly/nested/objects\")\n    {:ok, true}\n    iex> ExJSONPointer.resolve(data, \"/foo/1\", \"0#\")\n    {:ok, 1}","ref":"ExJSONPointer.html#resolve/3-examples"},{"type":"function","title":"ExJSONPointer.resolve_while/4","doc":"Resolve a JSON pointer while accumulating state during traversal.\n\nThis function allows you to track the traversal path and accumulate values as the JSON pointer\nis being resolved. It is designed to be useful for implementing operations that need context\nabout the traversal path, such as relative JSON pointers.","ref":"ExJSONPointer.html#resolve_while/4"},{"type":"function","title":"Parameters - ExJSONPointer.resolve_while/4","doc":"- `document`: The JSON document to be processed\n- `pointer`: A JSON pointer that identifies the location within the document\n- `acc`: An initial accumulator value that will be passed to the resolve function\n- `resolve_fun`: A function that receives the current value, reference token, and accumulated state\n  and returns either `{:cont, {new_value, new_acc}}` to continue or `{:halt, result}` to stop traversal\n\nThe `resolve_fun` receives three arguments:\n- The current value at the reference token\n- The current reference token being processed\n- A tuple containing the processing document and the current accumulator","ref":"ExJSONPointer.html#resolve_while/4-parameters"},{"type":"function","title":"Examples - ExJSONPointer.resolve_while/4","doc":"iex> data = %{\"a\" => %{\"b\" => %{\"c\" => [10, 20, 30]}}}\n    iex> init_acc = %{}\n    iex> fun = fn current, ref_token, {_document, acc} ->\n    ...>   {:cont, {current, Map.put(acc, ref_token, current)}}\n    ...> end\n    iex> {value, result} = ExJSONPointer.resolve_while(data, \"/a/b/c/0\", init_acc, fun)\n    iex> value\n    10\n    iex> result[\"c\"]\n    [10, 20, 30]","ref":"ExJSONPointer.html#resolve_while/4-examples"},{"type":"type","title":"ExJSONPointer.result/0","doc":"The result of resolving a JSON Pointer:\n* `{:ok, term()}` - the resolved value on success\n* `{:error, String.t()}` - when there is an error in pointer syntax or value not found","ref":"ExJSONPointer.html#t:result/0"},{"type":"extras","title":"ExJSONPointer","doc":"# ExJSONPointer\n\n[![hex.pm version](https://img.shields.io/hexpm/v/ex_json_pointer.svg?v=1)](https://hex.pm/packages/ex_json_pointer)\n\n<!-- MDOC !-->\n\nAn Elixir implementation of [RFC 6901](https://www.rfc-editor.org/rfc/rfc6901.html) JSON Pointer for locating specific values within JSON documents, and also supports [Relative JSON Pointer](https://datatracker.ietf.org/doc/html/draft-bhutton-relative-json-pointer-00) of the JSON Schema Specification draft-2020-12.","ref":"readme.html"},{"type":"extras","title":"Usage - ExJSONPointer","doc":"The JSON pointer string syntax can be represented as a JSON string:\n\n```elixir\niex> ExJSONPointer.resolve(%{\"a\" => %{\"b\" => %{\"c\" => \"hello\"}}}, \"/a/b/c\")\n{:ok, \"hello\"}\n\niex> ExJSONPointer.resolve(%{\"a\" => %{\"b\" => %{\"c\" => \"hello\"}}}, \"/a/b\")\n{:ok, %{\"c\" => \"hello\"}}\n\niex> ExJSONPointer.resolve(%{\"a\" => %{\"b\" => %{\"c\" => [1, 2, 3]}}}, \"/a/b/c\")\n{:ok, [1, 2, 3]}\n\niex> ExJSONPointer.resolve(%{\"a\" => %{\"b\" => %{\"c\" => [1, 2, 3]}}}, \"/a/b/c/2\")\n{:ok, 3}\n\niex> ExJSONPointer.resolve(%{\"a\" => [%{\"b\" => %{\"c\" => [1, 2]}}, 2, 3]}, \"/a/2\")\n{:ok, 3}\n\niex> ExJSONPointer.resolve(%{\"a\" => [%{\"b\" => %{\"c\" => [1, 2]}}, 2, 3]}, \"/a/0/b/c/1\")\n{:ok, 2}\n```\n\nor a URI fragment identifier:\n\n```elixir\niex> ExJSONPointer.resolve(%{\"a\" => %{\"b\" => %{\"c\" => \"hello\"}}}, \"#/a/b/c\")\n{:ok, \"hello\"}\n\niex> ExJSONPointer.resolve(%{\"a\" => %{\"b\" => %{\"c\" => \"hello\"}}}, \"#/a/b\")\n{:ok, %{\"c\" => \"hello\"}}\n\niex> ExJSONPointer.resolve(%{\"a\" => %{\"b\" => %{\"c\" => [1, 2, 3]}}}, \"#/a/b/c\")\n{:ok, [1, 2, 3]}\n\niex> ExJSONPointer.resolve(%{\"a\" => %{\"b\" => %{\"c\" => [1, 2, 3]}}}, \"#/a/b/c/2\")\n{:ok, 3}\n\niex> ExJSONPointer.resolve(%{\"a\" => [%{\"b\" => %{\"c\" => [1, 2]}}, 2, 3]}, \"#/a/2\")\n{:ok, 3}\n\niex> ExJSONPointer.resolve(%{\"a\" => [%{\"b\" => %{\"c\" => [1, 2]}}, 2, 3]}, \"#/a/0/b/c/1\")\n{:ok, 2}\n```\n\nSome cases that a JSON pointer that references a nonexistent value:\n\n```elixir\niex> ExJSONPointer.resolve(%{\"a\" => %{\"b\" => %{\"c\" => \"hello\"}}}, \"/a/b/d\")\n{:error, \"not found\"}\n\niex> ExJSONPointer.resolve(%{\"a\" => %{\"b\" => %{\"c\" => [1, 2, 3]}}}, \"/a/b/c/4\")\n{:error, \"not found\"}\n\niex> ExJSONPointer.resolve(%{\"a\" => %{\"b\" => %{\"c\" => \"hello\"}}}, \"#/a/b/d\")\n{:error, \"not found\"}\n\niex> ExJSONPointer.resolve(%{\"a\" => %{\"b\" => %{\"c\" => [1, 2, 3]}}}, \"#/a/b/c/4\")\n{:error, \"not found\"}\n```\n\nSome cases that a JSON pointer has some empty reference tokens, and link a `$ref` [test case](https://github.com/json-schema-org/JSON-Schema-Test-Suite/blob/main/tests/draft2020-12/ref.json#L1023) from JSON Schema Test Suite(draft 2020-12) for reference.\n\n```elixir\niex> ExJSONPointer.resolve(%{\"\" => %{\"\" => 1}}, \"/\")\n{:ok, %{\"\" => 1}}\n\niex> ExJSONPointer.resolve(%{\"\" => %{\"\" => 1}}, \"//\")\n{:ok, 1}\n\niex> ExJSONPointer.resolve(%{\"\" => %{\"\" => 1, \"b\" => %{\"\" => 2}}}, \"//b\")\n{:ok, %{\"\" => 2}}\n\niex> ExJSONPointer.resolve(%{\"\" => %{\"\" => 1, \"b\" => %{\"\" => 2}}}, \"//b/\")\n{:ok, 2}\n\niex> ExJSONPointer.resolve(%{\"\" => %{\"\" => 1, \"b\" => %{\"\" => 2}}}, \"//b///\")\n{:error, \"not found\"}\n```\n\nInvalid JSON pointer syntax:\n\n```elixir\niex> ExJSONPointer.resolve(%{\"a\" =>%{\"b\" => %{\"c\" => [1, 2, 3]}}}, \"a/b\")\n{:error, \"invalid JSON pointer syntax\"}\n\niex> ExJSONPointer.resolve(%{\"a\" =>%{\"b\" => %{\"c\" => [1, 2, 3]}}}, \"##/a\")\n{:error, \"invalid JSON pointer syntax\"}\n\n```","ref":"readme.html#usage"},{"type":"extras","title":"Relative JSON Pointer - ExJSONPointer","doc":"This library also supports [Relative JSON Pointer](https://datatracker.ietf.org/doc/html/draft-bhutton-relative-json-pointer-00) of the JSON Schema Specification draft-2020-12 which allows you to reference values relative to a specific location within a JSON document.\n\nA relative JSON pointer consists of:\n- A non-negative integer (prefix) that indicates how many levels up to traverse\n- An optional index manipulation (+N or -N) for array elements\n- An optional JSON pointer to navigate from the referenced location\n\n```elixir\n# Sample data\niex> data = %{\"foo\" => [\"bar\", \"baz\"], \"highly\" => %{\"nested\" => %{\"objects\" => true}}}\niex> ExJSONPointer.resolve(data, \"/foo/1\", \"0\") # Get the current value (0 levels up)\n{:ok, \"baz\"}\n# Get the parent array and access its first element (1 level up, then to index 0)\niex> ExJSONPointer.resolve(data, \"/foo/1\", \"1/0\")\n{:ok, \"bar\"}\n# Get the previous element in the array (current level, index - 1)\niex> ExJSONPointer.resolve(data, \"/foo/1\", \"0-1\")\n{:ok, \"bar\"}\n# Go up to the root and access a nested property\niex> ExJSONPointer.resolve(data, \"/foo/1\", \"2/highly/nested/objects\")\n{:ok, true}\n# Get the index of the current element in its array\niex> ExJSONPointer.resolve(data, \"/foo/1\", \"0#\")\n{:ok, 1}\n\n# Get the key name of a property in an object\niex> data2 = %{\"features\" => [%{\"name\" => \"environment friendly\", \"url\" => \"http://example.com\"}]}\niex> ExJSONPointer.resolve(data2, \"/features/0/url\", \"1/name\")\n{:ok, \"environment friendly\"}\niex> ExJSONPointer.resolve(data2, \"/features/0/url\", \"2#\")\n{:ok, \"features\"}\n```\n\nPlease see the test cases for more examples.","ref":"readme.html#relative-json-pointer"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.3"}}